# Wobbly Balls
# Copyright (C) 2010 Jeffrey Armstrong <jeff@rainbow-100.com>
# http://jeff.rainbow-100.com/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# A full version of the license terms is available in LICENSE.

import data
import pygame
from board import Rocker, ColumnGuides, Danger
from constants import *
from physics import Physics
from level import build_levels, total_drops, LevelAnnounce
from score import ScoreBoard, InfoBoard
from loss import lose

def game(screen):
    fbkgd = data.load('bkgd.png')
    bkgd = pygame.image.load(fbkgd)
    fbkgd.close()
    screen.blit(bkgd,(0,0))
    pygame.display.flip()

    # Initialize clock
    clock = pygame.time.Clock()

    # Setup the board
    rocker = Rocker()
    board_spr = pygame.sprite.RenderUpdates(rocker, rocker.guides)
    
    # Setup the sprite rendering for blocks
    blocks_spr = rocker.block_rendering
    
    # Setup the play blocks (the ones to be placed)
    play_spr = pygame.sprite.RenderUpdates()
    
    # Setup the auxiliary sprites
    score = ScoreBoard()
    infob = InfoBoard()
    danger = Danger()
    aux_spr = pygame.sprite.RenderUpdates(score, infob, danger)

    # Initialize the physics
    physics = Physics(rocker)    

    # Initialize levels
    levels = build_levels()
    current_level = 0
    level_announce = None

    play_group = None
    play_clock = 0
    play_ball_count = 3

    angle_clock = 0
    #rocker.angle = 30

    playing = True
    loss = False
    
    left = False
    right = False

    running = True
    while running:
        clock.tick(FPS)
        
        # Clear the sprites
        board_spr.clear(screen,bkgd)
        blocks_spr.clear(screen,bkgd)
        play_spr.clear(screen,bkgd)
        aux_spr.clear(screen,bkgd)             
        
        # Create a play group if necessary
        if play_group == None and playing:
            play_group = rocker.new_play_group(levels[current_level])
            play_clock = pygame.time.get_ticks()
            infob.set_playgroup(play_group)
        
        # Too long?  Random play group drop
        if pygame.time.get_ticks() - play_clock > TIME_LIMIT_PG and playing and play_group != None:
            rocker.random_drop_play_group()
            levels[current_level].drop()
            play_group = None
            
        # Game logic!
        for e in pygame.event.get():
            
            if play_group and playing:
                if e.type == pygame.KEYDOWN:
                    if e.key == pygame.K_LEFT or e.key == pygame.K_a:
                        left = True
                    elif e.key == pygame.K_RIGHT or e.key == pygame.K_d:
                        right = True
                    elif e.key == pygame.K_DOWN or e.key == pygame.K_s:
                        if rocker.drop_play_group():
                            play_group = None
                            levels[current_level].drop()
                    elif e.key == pygame.K_ESCAPE:
                        playing = False
                        running = False
                elif e.type == pygame.KEYUP:
                    if e.key == pygame.K_LEFT or e.key == pygame.K_a:
                        left = False
                    elif e.key == pygame.K_RIGHT or e.key == pygame.K_d:
                        right = False
                    

        # Movement
        if play_group:
            if left:
                play_group.left()
            elif right:
                play_group.right()

        # Update the physics
        physics.update()
        
        # Angle stuff
        if abs(rocker.angle) > 30.0:
            danger.on()
        else:
            danger.off()  
            
        if abs(rocker.angle) > 45.0:
            running = False
            loss = True
        
        # Check level completion
        if levels[current_level].win():
            current_level = current_level + 1
            rocker.empty_all()
            level_announce = LevelAnnounce(current_level+1)
            aux_spr.add(level_announce)
            
        # Fun Level Announcements
        if level_announce:
            if level_announce.done:
                aux_spr.remove(level_announce)
                level_announce = None
        
        # Update the scoreboard
        score.level = levels[current_level]
        score.drops = total_drops(levels)
        
        # Debugging
        #if pygame.time.get_ticks() - angle_clock > 1000:
        #    rocker.set_angle(rocker.angle - 1)
        #    angle_clock = pygame.time.get_ticks()

        
        
        # Dirty sprites drawing
        board_spr.update()
        blocks_spr.update()
        play_spr.update()
        aux_spr.update()
        
        dirty = board_spr.draw(screen) + \
                blocks_spr.draw(screen) + \
                play_spr.draw(screen) + \
                aux_spr.draw(screen)
        pygame.display.update(dirty)
        
    # Handle Losing here
    if loss:
        board_spr.add(blocks_spr,play_spr,aux_spr)
        lose(screen,bkgd,board_spr)
    

    
