# Wobbly Balls
# Copyright (C) 2010 Jeffrey Armstrong <jeff@rainbow-100.com>
# http://jeff.rainbow-100.com/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# A full version of the license terms is available in LICENSE.

import pygame
import random
from constants import *
import data

LOSS_TEXT = ['Loser!', 'TILT', 'Fail', 'Defeat']

class LoseText(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        i = random.randint(0,len(LOSS_TEXT)-1)
        
        font = pygame.font.Font(data.filepath(SCORE_FONT_NAME),4*SCORE_FONT_SIZE)
        render = font.render(LOSS_TEXT[i],1,pygame.Color('white'))
        
        self.image = pygame.Surface(render.get_size())
        self.image.fill(pygame.Color('black'))
        self.image.set_colorkey(pygame.Color('black'))
        self.image.blit(render,(0,0))
        
        self.rect = pygame.Rect(self.image.get_rect())
        self.rect.center = (SCREEN_X/2,SCREEN_Y/2)
        
        self.image.set_alpha(0)
        self.start = pygame.time.get_ticks()
        
    def update(self):
        alpha = float(pygame.time.get_ticks() - self.start)/float(LOSE_TIME)
        alpha = min(255,int(alpha*255.0))

        self.image.set_alpha(alpha)
        
class Flash(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        self.image = pygame.Surface((SCREEN_X,SCREEN_Y))
        self.image.set_colorkey(pygame.Color('black'))
        self.image.fill(pygame.Color('white'))
        
        self.rect = pygame.Rect(0,0,SCREEN_X,SCREEN_Y)
        
        self.image.set_alpha(0)
        self.start = pygame.time.get_ticks()
        self.end = None
    
    def done(self):
        if self.end:
            return pygame.time.get_ticks() - self.end > FLASH_TIME
        else:
            return False
    
    def update(self):
        alpha = 0
        if self.start:
            fraction = float(pygame.time.get_ticks() - self.start)/float(FLASH_TIME)
            alpha = min(255,int(fraction*255.0))
            if fraction >= 1:
                self.start = None
                self.end = pygame.time.get_ticks()
        elif self.end:
            fraction = 1.0 - float(pygame.time.get_ticks() - self.end)/float(FLASH_TIME)
            alpha = min(255,int(fraction*255.0))
        
        self.image.set_alpha(alpha)

class Pak(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        font = pygame.font.Font(data.filepath(SCORE_FONT_NAME),BRAG_FONT_SIZE)
        render = font.render('Press Any Key to Continue...',1,pygame.Color('white'))
        
        self.image = pygame.Surface(render.get_size())
        self.image.set_colorkey(pygame.Color('black'))
        self.image.fill(pygame.Color('black'))
        self.image.blit(render,(0,0))
        
        self.rect = pygame.Rect((0,0),self.image.get_size())
        self.rect.center = (SCREEN_X/2,SCREEN_Y-10-self.rect.height/2)
    
def lose(screen,bkgd,oldsprites):
    
    flash = Flash()
    lt = LoseText()
    pak = Pak()
    sprites = pygame.sprite.RenderUpdates(flash, lt)
    
    clock = pygame.time.Clock()
    
    loss_startup = pygame.time.get_ticks()
    key_check = False
    running = True
    while running:
        clock.tick(FPS)
        
        oldsprites.clear(screen,bkgd)
        sprites.clear(screen,bkgd)
        
        if pygame.time.get_ticks() - loss_startup > LOSE_TIME:
            key_check = True
            sprites.add(pak)
        else:
            pygame.event.clear()
            
        if key_check:
            for e in pygame.event.get():
                if e.type == pygame.KEYDOWN:
                    running = False
            
        if flash:
            if flash.done():
                sprites.remove(flash)
                flash = None
        
        sprites.update()
        
        dirty = oldsprites.draw(screen) + \
                sprites.draw(screen)
        pygame.display.update(dirty)
        