# Wobbly Balls
# Copyright (C) 2010 Jeffrey Armstrong <jeff@rainbow-100.com>
# http://jeff.rainbow-100.com/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# A full version of the license terms is available in LICENSE.

import pygame
from constants import *
import data
from block import GBlock
import random

class Selection(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)        
        self.selection = 1
        
        self.rect = pygame.Rect(0,0,64,64)
        
        self.image = pygame.Surface((64,64))
        self.image.set_colorkey(pygame.Color('black'))
        self.image.fill(pygame.Color('black'))
        
        pygame.draw.ellipse(self.image, pygame.Color(0,255,0,255), self.rect, 0)
        pygame.draw.ellipse(self.image, pygame.Color(255,0,0,255), self.rect, 12)
    
    def up(self):
        self.selection = max(1,self.selection - 1)
        
    def down(self):
        self.selection = min(3,self.selection + 1)
    
    def position(self):
        x = 263
        y = 239
        if self.selection == 2:
            y = 321
        elif self.selection == 3:
            y = 399
        self.rect.center = (x,y)
    
    def update(self):
        self.position()
    
class FunBlock(GBlock):
    def __init__(self):
        GBlock.__init__(self)
        
        self.rect.center = (random.randint(0,SCREEN_X),-1*BLOCK_Y)
        self.duration = random.randint(1000,4000)
        self.start = pygame.time.get_ticks()

    def done(self):
        return self.rect.top > SCREEN_Y
        
    def update(self):
        dist = float(SCREEN_Y+2*BLOCK_Y)
        fraction = float(pygame.time.get_ticks()-self.start)/self.duration
        self.rect.bottom = -1*BLOCK_Y + int(dist*fraction)

class Instructions(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self)
        
        fbkgd = data.load('instructions.png')
        ibkgd = pygame.image.load(fbkgd)
        fbkgd.close()
        
        self.image = pygame.Surface(ibkgd.get_size())
        self.image.set_colorkey(pygame.Color(255,0,0,255))
        self.image.fill(pygame.Color(255,0,0,255))
        self.image.blit(ibkgd,(0,0))
        
        self.alpha = 0
        self.image.set_alpha(0)
        
        self.rect = self.image.get_rect()
        self.rect.center = (SCREEN_X/2,SCREEN_Y/2)
        
        self.start = False
        self.end = False

    def on(self):
        self.start = True
        self.end = False
    
    def off(self):
        self.start = False
        self.end = True
    
    def done(self):
        return self.alpha <= 0
    
    def update(self):
        if self.start:
            self.alpha = min(self.alpha+10,255) 
            self.image.set_alpha(self.alpha)
        elif self.end:
            self.alpha = max(0,self.alpha-10) 
            self.image.set_alpha(self.alpha)
            if self.alpha <= 0:
                self.end = False

def menu(screen):
    
    screen.fill(pygame.Color('black'))
    
    fbkgd = data.load('mainmenu.png')
    ibkgd = pygame.image.load(fbkgd)
    fbkgd.close()
    
    bkgd = pygame.Surface((SCREEN_X,SCREEN_Y))
    bkgd.set_colorkey(pygame.Color(0,0,255,255))
    bkgd.blit(ibkgd,(0,0))
    
    salpha = 0
    bkgd.set_alpha(0)
    while salpha < 100:
        screen.blit(bkgd,(0,0))
        pygame.display.flip()
        salpha = salpha + 3
        bkgd.set_alpha(salpha)
        pygame.time.wait(4)
        
    bkgd.set_alpha(255)
    screen.blit(bkgd,(0,0))
    pygame.display.flip()

    play = False
    quit = False
    running = True
    selected = False
    
    select = Selection()
    inst = Instructions()
    sprites = pygame.sprite.RenderUpdates(inst,select)

    fun = pygame.sprite.RenderUpdates()
    
    clock = pygame.time.Clock()

    while running: 
        clock.tick(FPS)
        
        sprites.clear(screen,bkgd)
        fun.clear(screen,bkgd)
        
        for e in pygame.event.get():
            if e.type == pygame.KEYDOWN:
                if inst.start:
                    inst.off()
                    pygame.event.clear()
                else:
                    if e.key == pygame.K_UP or e.key == pygame.K_w:
                        select.up()
                    elif e.key == pygame.K_DOWN or e.key == pygame.K_s:
                        select.down()
                    elif e.key == pygame.K_SPACE or e.key == pygame.K_RETURN:
                        selected = True
                    elif e.key == pygame.K_ESCAPE:
                        selected = True
                        select.selection = 3

        if selected:
            if select.selection == 1:
                running = False
                play = True
            elif select.selection == 3:
                running = False
                quit = True
            else:
                inst.on()
                
            selected = False
        
        if len(fun.sprites()) < 18:
            fun.add(FunBlock())
            
        for x in fun.sprites():
            if x.done():
                fun.remove(x)
        
        sprites.update()
        fun.update()
        dirty = sprites.draw(screen) + fun.draw(screen)
        pygame.display.update(dirty)
        
    pygame.event.clear()
    
    salpha = 0
    sclear = pygame.Surface((SCREEN_X,SCREEN_Y))
    sclear.fill(pygame.Color('black'))
    sclear.set_alpha(0)
    while salpha <= 128:
        screen.blit(sclear,(0,0))
        pygame.display.flip()
        #pygame.time.wait(2)
        salpha = salpha + 1
        sclear.set_alpha(salpha)
    
    return play,quit
