# Wobbly Balls
# Copyright (C) 2010 Jeffrey Armstrong <jeff@rainbow-100.com>
# http://jeff.rainbow-100.com/
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# A full version of the license terms is available in LICENSE.

import math
import pygame
from constants import *

class Physics:
    
    def __init__(self,rocker):
        self.target = 0.0
        self.rocker = rocker
        
        self.velocity = 0.0
        
        self.time = pygame.time.get_ticks()
        self.check_flippy = False
        
    def balanced_dx(self):
        torques = 0.0
        for c in self.rocker.columns:
            for b in c.blocks:
                cx = b.rect.centerx - self.rocker.rect.centerx
                torques = torques - b.weight*WEIGHT_SCALE*cx
        #print torques
        return torques/SPRING_CONSTANT/self.rocker.radius

    def acceleration(self):
        da = self.target - self.rocker.angle
        
        return -POS_ACCEL*da - VEL_ACCEL*self.velocity

    def update(self):
        """Internally updates the physics states and appropriately passes
        the new angle into the rocker."""
        circumference = 2.0*math.pi*self.rocker.radius
        self.target = self.balanced_dx() # /circumference*360.0
        if abs(self.target) > 45.0:
            self.target = self.target*45.0/abs(self.target)

        newtime = pygame.time.get_ticks()
        dt = newtime - self.time
        dt = float(dt)/1000.0
        
        if self.acceleration() != 0.0:
            pass
        
        #self.velocity = 0.03*(self.rocker.angle- self.target) 
        self.velocity = self.velocity + dt*self.acceleration()
        
        da = dt*self.velocity
        if abs(da) > 10.0/60.0:
            da = da*10.0/60.0/abs(da)
        newangle = self.rocker.angle-da
        
        #if abs(newangle-self.target) < 0.001 and self.velocity*dt < 0.001:
        #    self.velocity = 0
        #    newangle = self.target
        
        if abs(newangle) > 50.0:
            print '!!!!WARNING!!!!: Tgt %f newangle %f vel %f rocker %f' % (self.target,newangle,self.velocity,self.rocker.angle)
            newangle = 46.0
            self.velocity = 0.0
        
        self.rocker.set_angle(newangle)

        return self.rocker.angle
